from cloudmesh.shell.command import command
from cloudmesh.shell.command import PluginCommand
from cloudmesh.common.console import Console
from cloudmesh.common.util import path_expand
from pprint import pprint
from cloudmesh.common.debug import VERBOSE
from cloudmesh.pearl.Pearl import Pearl
from cloudmesh.common.util import yn_choice
from cloudmesh.common.Shell import Shell
import os

PYTHON="python3"

class PearlCommand(PluginCommand):

    # noinspection PyUnusedLocal
    @command
    def do_pearl(self, args, arguments):
        """
        ::

          Usage:
                pearl user USER
                pearl key KEY
                pearl queue
                pearl ssh [COMMAND...]
                pearl batch SCRIPT
                pearl run [--cpu=CPU] [--gpu=GPU] [--output=OUTPUT]
                pearl sync put [DIR]
                pearl sync get [DIR]
                pearl fuse DIR
                pearl venv [VENV] [--python=PYTHON] [-y|-n]
                pearl install
                pearl notebook NAME [--cpu=CPU] [--gpu=GPU] [--duration=DURATION] [--force]
                pearl doc
                pearl module
                pearl info
                pearl verbose [ON]

          Interfacing with pearl

          Arguments:
              USER     the username
              SCRIPT   script to be executed
              DIR      the DIRECTORY to be synced to pearl
              OPTIONS  options
              VENV     The virtual python env

        """

        pearl = Pearl()

        arguments.yes = arguments["-y"]
        arguments.no  = arguments["-n"]
        arguments.python = arguments["--python"] or "python"
        arguments.gpu = arguments["--gpu"]
        arguments.cpu = arguments["--cpu"]
        arguments.VENV = arguments.VENV or "PEARL"
        arguments.FORCE = arguments["--force"]
        arguments.DIR = arguments.DIR or "."

        VERBOSE(arguments)
        if arguments.doc:

            Shell.browser('https://pearl-cluster.readthedocs.io/en/latest/')

        elif arguments.user:

            pearl.set_user(arguments.USER)
            return

        try:
            pearl.check_user()
        except ValueError:
            Console.error("Please set the user with\n")
            Console.msg("   cms pearl user YOURUSERID\n")

        if arguments.info:
            pearl.info()

        elif arguments.verbose:
            pearl.set_verbose(arguments.ON)

        elif arguments.key:
            pearl.set_key(arguments.KEY)

        elif arguments.venv:

            if not arguments.VENV.startswith("~"):
                arguments.VENV = f"~/{arguments.VENV}"
            arguments.VENV = path_expand(arguments.VENV)


            question = f"Do you like to delete and create the venv {arguments.VENV}"

            if arguments.yes:
                create = True
            elif arguments.no:
                create = False
            elif yn_choice(question):
                create = True
            else:
                create = False

            if create:
                Console.ok(f"Creating venv {arguments.VENV}")
                os.system(f"rm -rf {arguments.VENV}")
                os.system(f"python -m venv {arguments.VENV}")
            else:
                Console.error(f"Skipping the creation of the venv {arguments.VENV}")

        elif arguments.install:

            pearl.ssh(execute=f"python3 -m venv {arguments.VENV}")

            # pearl.ssh(execute="pip install cloudmesh-pearl")
            # pearl.ssh(execute="pip install cloudmesh-pearl")
            # pearl.ssh(execute="pearl")

        elif arguments.module:

            pearl.module()

        elif arguments.info:

            pearl.info()

        elif arguments.queue:

            pearl.queue()

        elif arguments.notebook:
            # pearl notebook NAME [--cpu=CPU] [--gpu=GPU] [--output=OUTPUT]
            pearl.notebook(name=arguments.NAME,
                           cpu=arguments.CPU,
                           gpu=arguments.GPU,
                           duration=arguments.DURATION,
                           force=arguments.FORCE)

        elif arguments.ssh:

            if len (arguments.COMMAND) > 0:
                arguments.COMMAND = ' '.join(arguments.COMMAND)
            else:
                arguments.COMMAND = None
            pearl.ssh(execute=arguments.COMMAND)

        elif arguments.run:

            pearl.run(cpu=arguments.cpu, gpu=arguments.gpu)

        elif arguments.sync and arguments.put:

            pearl.sync_put(arguments.DIR)

        elif arguments.sync and arguments.get:

            pearl.sync_get(arguments.DIR)

        elif arguments.batch:

            pearl.batch(arguments.SCRIPT)

        else:
            Console.error("check your usage")

        """
        USER
            pearl batch SCRIPT
            pearl run [OPTIONS]
            pearl fuse DIR
            pearl
        """

        return ""



from cloudmesh.common.variables import Variables
import os
from cloudmesh.common.util import path_expand
from cloudmesh.common.console import Console
import textwrap
from cloudmesh.common.util import writefile

class Pearl(object):

    def __init__(self):

        self.variables = Variables()
        self.host = "ui.pearl.scd.stfc.ac.uk"
        try:
            self.username = self.variables["pearl.username"]
        except:
            Console.error("Username not set")
        try:
            self.key = self.variables["pearl.key"]
        except:
            Console.error("Key not set")
        try:
            self.verbose = self.variables["pearl.verbose"]
        except:
            self.variables["pearl.verbose"] = self.verbose = False


    def check_user(self):
        if self.username is None:
            raise ValueError("Your user name for perl is not set")

    def _execute(self, command):
        command = f"ssh -i {self.key} {self.username}@{self.host} '{command}'"
        if self.verbose:
            print (command)
        os.system(command)

    def module(self):
        self.ssh(execute="source /etc/profile; module avail")

    def set_verbose(self, on=None):
        if on is None or on:
            self.variables["pearl.verbose"] = True
        else:
            self.variables["pearl.verbose"] = on.lower() in ["1", "true", "on"]
        if self.variables["pearl.verbose"]:
            os.system("cms debug on")
        else:
            os.system("cms debug off")

    def set_user(self, username):
        if "@" in username:
            username = username.split("@")[0]
        self.username = self.variables["pearl.username"] = username

    def set_key(self, key):
        self.key = path_expand(key)
        self.variables["pearl.key"] = self.key

    def queue(self):
        self._execute("squeue")

    def ssh(self, execute=None, shell=True):
        if execute is not None:
            if not shell:
                execute = f'"{execute}"'
                command = f"ssh -i {self.key} {self.username}@{self.host} {execute}"
            else:
                command = f'echo "{execute}" |'\
                    f' ssh -i /home/green/.ssh/id_rsa.pub {self.username}@{self.host} /bin/bash -l'
        else:
            command = f"ssh -i {self.key} {self.username}@{self.host}"
        if self.verbose:
            print (command)
        os.system(command)

    def batch(self, script):
        self.sync_put(".")
        self.ssh(f"sbatch notebooks/{script}")
        self.sync_get(".")

    def generate_notebook_batch(self, notebook=None, name="script", cpu=1, gpu=1, duration="00:00:01"):
        stem = notebook.replace(".ipynb", "")
        duration = duration or "00:00:01"

        batch_script = textwrap.dedent(f"""
        #!/bin/bash                                                                                                              

        #SBATCH -p pearl # partition (queue)                                                                                     
        #SBATCH --job-name={name}
        #SBATCH -n {cpu} # number of CPU cores
        #SBATCH --gres=gpu:{gpu}
        #SBATCH -t {duration} # time (D-HH:MM)
        #SBATCH --chdir=notebooks
        
        echo "#"
        echo "# PYTHON"
        echo "#"
        source ~/ENV3/bin/activate
        which python
        python --version
        echo "#"
        echo "# NVIDIA"
        echo "#"
        nvidia-smi
        echo "#"        
        # 
        # COMMANDS
        #
        """).strip() + "\n"
        end_script = "\n#\n"

        notebook_script = f"jupyter nbconvert --allow-errors --execute --to notebook  --output={stem}-output.ipynb  {notebook}"
        return batch_script + notebook_script + end_script

    def notebook(self, name=None, cpu=None, gpu=None, duration=None, force=False):
        gpu = gpu or 1
        cpu = cpu or 1
        duration = duration or "00:00:01"
        stem = name.replace(".ipynb", "")
        script = name.replace(".ipynb", ".sh")

        if force or not os.path.exists(script) :
            content = self.generate_notebook_batch(
                notebook=name,
                name=stem,
                cpu=cpu,
                gpu=gpu,
                duration=duration
            )
            writefile(script, content=content)
        self.batch(script)


    def run(self, cpu=None, gpu=None):
        gpu = gpu or 1
        cpu = cpu or 1
        command = f"srun -n {cpu} --gres=gpu:{gpu} --pty /bin/bash"
        srun = f"ssh -i {self.key} -t {self.username}@{self.host} '{command}'"
        os.system(srun)

    def ls(self, directory):
        self.ssh(f"ls -lisa {directory}")

    def sync_put(self, directory):
        command = f"rsync -r {directory} {self.username}@{self.host}:notebooks/{directory}"
        if self.verbose:
            print (command)
        os.system(command)
        self.ls(f"notebooks/{directory}")

    def sync_get(self, directory):
        remote = f"notebooks/{directory}"
        local = directory
        if local == ".":
            local = os.getcwd()
        command = f"rsync -r {self.username}@{self.host}:{remote} {local}"
        if self.verbose:
            print (command)
        os.system(command)
        os.system(f"ls -lisa {directory}")

    def fuse(self, DIR):
        pass

    def info(self):
        print()
        print("Username:", self.username)
        print("Key:     ", self.key)
        print("Verbose: ", self.verbose)
        print("Queues")
        print()
        self.ssh(execute="echo; sinfo; echo; squeue -l")
        print()
